--
-- Rubbish by Ludovicus (LuisOrtiz@Verizon.NET)
--
-- Merchant Sales Management Functions
--
-- This addon is dedidicated to Harry Wasylyk & Larry Hansen, the Canadian inventors of the disposable green polyethylene garbage bag.


function Rubbish:Merchant_Init()
	Rubbish:dprint("Rubbish:Merchant_Init()")
	Rubbish:RegisterEvent("MERCHANT_SHOW");
	Rubbish:AddMerchantTab()
end


function Rubbish:MERCHANT_SHOW(event)
	if RubbishDB["AlwaysSellJunk"] then
		Rubbish:GenerateSellList()
	end
end


function Rubbish:AddMerchantTab()
	-- Be a little paranoid.  We want to add a tab, so we need to see how many there are!
	local tab = MerchantFrame.numTabs + 1
	Rubbish.SellTab = CreateFrame("Button","MerchantFrameTab"..tab,MerchantFrame,"CharacterFrameTabButtonTemplate",tab)
	_G["MerchantFrameTab"..tab] = Rubbish.SellTab
	Rubbish.SellTab:SetText("Sell")
	Rubbish.SellTab:SetPoint("LEFT","MerchantFrameTab"..(tab-1),"RIGHT",-16,0)
	PanelTemplates_SetNumTabs(MerchantFrame, tab);
	-- Override the OnClick script for all the tabs so we can paginate cleanly
	for i=1, tab do
		local tabButton = _G["MerchantFrameTab"..i]
		tabButton:SetScript("OnClick", function()
			PanelTemplates_SetTab(MerchantFrame, i);
			Rubbish:MerchantFrame_Update();
		end);
	end
	Rubbish.SellTabIdx = tab
	-- Create the checkbox that controls if we automatically sell gray items when the sell tab is selected
	Rubbish.AlwaysJunkCheckbox = CreateFrame("CheckButton","RubbishAlwaysJunkCheckbox",MerchantFrame, "UICheckButtonTemplate")
	Rubbish.AlwaysJunkCheckbox:ClearAllPoints()
	Rubbish.AlwaysJunkCheckbox:SetPoint("BOTTOMLEFT","MerchantFrame","BOTTOMLEFT",17,59)
	_G["RubbishAlwaysJunkCheckboxText"]:SetText("Autosell Junk")
	Rubbish.AlwaysJunkCheckbox:SetChecked(RubbishDB["AlwaysSellJunk"])
	Rubbish.AlwaysJunkCheckbox:SetScript("PostClick", function (self, button, down)
		local status = self:GetChecked() ~= nil
		if status ~= RubbishDB["AlwaysSellJunk"] then
			Rubbish:say("Always Sell Junk set to ", status)
			RubbishDB["AlwaysSellJunk"] = status
		end
	end)
	Rubbish.AlwaysJunkCheckbox:Hide()
end


function Rubbish:MerchantFrame_Update()
	if ( MerchantFrame.selectedTab == Rubbish.SellTabIdx) then
		-- Set the handlers for each button to our own
		if Rubbish.Orig_MerchantItem1ItemButton_OnEnter == nil then
			-- Save only once, in case the tab gets clicked more than once!
			Rubbish.Orig_MerchantItem1ItemButton_OnEnter = MerchantItem1ItemButton:GetScript("OnEnter")
			Rubbish.Orig_MerchantItem1ItemButton_OnClick = MerchantItem1ItemButton:GetScript("OnClick")
			Rubbish.Orig_MerchantItem1ItemButton_OnDragStart = MerchantItem1ItemButton:GetScript("OnDragStart")
			Rubbish:say("MerchantFrame_Update: cached _Orig")
		end
		for i=1, BUYBACK_ITEMS_PER_PAGE do
			local itemButton = _G["MerchantItem"..i.."ItemButton"];
			itemButton.UpdateTooltip = MerchantJunkItemButton_OnEnter
			itemButton:SetScript("OnEnter",MerchantJunkItemButton_OnEnter)
			itemButton:SetScript("OnClick",MerchantJunkItemButton_OnClick)
			itemButton:SetScript("OnDragStart",nil)			
		end	
		Rubbish:MerchantFrame_UpdateJunkInfo()
	else
		Rubbish.AlwaysJunkCheckbox:Hide()
		-- ReSet the handlers for each button to the originals
		Rubbish:say("MerchantFrame_Update: restored _Orig")
		for i=1, BUYBACK_ITEMS_PER_PAGE do
			local itemButton = _G["MerchantItem"..i.."ItemButton"];
			itemButton.UpdateTooltip = Rubbish.Orig_MerchantItem1ItemButton_OnEnter
			itemButton:SetScript("OnEnter",Rubbish.Orig_MerchantItem1ItemButton_OnEnter)
			itemButton:SetScript("OnClick",Rubbish.Orig_MerchantItem1ItemButton_OnClick)
			itemButton:SetScript("OnDragStart",Rubbish.Orig_MerchantItem1ItemButton_OnDragStart)			
		end
		Rubbish.Orig_MerchantItem1ItemButton_OnEnter = nil
		Rubbish.Orig_MerchantItem1ItemButton_OnClick = nil
		Rubbish.Orig_MerchantItem1ItemButton_OnDragStart = nil
		
		PanelTemplates_SetTab(MerchantFrame, MerchantFrame.selectedTab);
		MerchantFrame_Update()
	end
	
end


function MerchantJunkItemButton_OnEnter(button)
	GameTooltip:SetOwner(button, "ANCHOR_RIGHT");
	GameTooltip:SetBagItem(button.bag,button.slot)
	MerchantFrame.itemHover = button:GetID();
	ShowBuybackSellCursor(button:GetID());
end


function MerchantJunkItemButton_OnClick(self, button)
	UseContainerItem(self.bag,self.slot)
end


function Rubbish:VendorOrNot(bag,slot)
	local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag,slot);
	if texture == nil then return nil; end
	local itemID = GetContainerItemID(bag,slot)
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID)
	local suggestion = AucAdvanced and AucAdvanced.Modules.Util.ItemSuggest.itemsuggest(itemLink,itemCount)
	if suggestion == "Vendor" then
--		Rubbish:say("VendorOrNot Vendor",bag,slot,itemCount * itemSellPrice,itemName)
		return itemRarity , itemCount * itemSellPrice , itemName
	elseif suggestion ~= nil then
--		Rubbish:say("VendorOrNot Vendor reject",bag,slot,itemName)
		return nil
	end
	if itemRarity < 0 then
		return nil;
	end
--	Rubbish:say("VendorOrNot itemRarity",bag,slot,itemCount * itemSellPrice,itemName)
	return itemRarity , itemCount * itemSellPrice * (itemRarity+1) , itemName
end


local Rubbish_SortBySell
function Rubbish_SortBySell(a,b)
	if (a[3] < b[3]) then
		return true  -- primary sort
	elseif (a[3] == b[3]) then
		return (a[4] < b[4]) -- secondary sort
	else
		return false -- A > B by both conditions
	end
end


function Rubbish:GenerateSellList()
	local sell_list = {}
	for bag=BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		local slots = GetContainerNumSlots(bag)
		if slots then
			for slot=1,slots do
				local itemRarity, itemPrice , itemName = Rubbish:VendorOrNot(bag,slot)
				if itemRarity then
					if itemRarity == 0 and RubbishDB["AlwaysSellJunk"] then
						UseContainerItem(bag,slot)
					else				
						table.insert(sell_list,{bag, slot, itemRarity, itemPrice, itemName})
					end
				end
			end
		end
	end
	table.sort(sell_list,Rubbish_SortBySell)
	return sell_list
end




function Rubbish:MerchantFrame_UpdateJunkInfo()
	MerchantNameText:SetText("Junk");
	MerchantFramePortrait:SetTexture("Interface\\InventoryItems\\WoWUnknownItem01");




	-- Show Junk specific items

	Rubbish.AlwaysJunkCheckbox:Show()


	-- Position Junk items
	local sellItems = Rubbish:GenerateSellList()
	local numSellItems = #sellItems
	Rubbish:say("MerchantFrame_UpdateJunkInfo selling", numSellItems)
	for i=1, BUYBACK_ITEMS_PER_PAGE do
		local itemButton = _G["MerchantItem"..i.."ItemButton"];
		local buybackButton = _G["MerchantItem"..i];
		_G["MerchantItem"..i.."AltCurrencyFrame"]:Hide();
		if ( i <= numSellItems) then
			local texture, itemCount, locked, quality, readable = GetContainerItemInfo(sellItems[i][1],sellItems[i][2])
			local itemID = GetContainerItemID(sellItems[i][1],sellItems[i][2])
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID)
			Rubbish:say("MerchantFrame_UpdateJunkInfo selling", i, sellItems[i][1],sellItems[i][2],sellItems[i][3],sellItems[i][4],sellItems[i][5])
			_G["MerchantItem"..i.."Name"]:SetText(itemName);
			SetItemButtonCount(itemButton, itemCount);
			SetItemButtonStock(itemButton, itemCount);
			SetItemButtonTexture(itemButton, itemTexture);


			_G["MerchantItem"..i.."MoneyFrame"]:Show();
			MoneyFrame_Update("MerchantItem"..i.."MoneyFrame", itemSellPrice*itemCount);
			itemButton:SetID(i);
--			itemButton.link = itemLink
--			itemButton.texture = itemTexture
			itemButton.bag = sellItems[i][1]
			itemButton.slot = sellItems[i][2]
			itemButton.UpdateTooltip = MerchantJunkItemButton_OnEnter
--			itemButton:UpdateTooltip()
			itemButton:Show();


			if (itemRarity == 0 ) then
				SetItemButtonDesaturated(buybackButton,1)
				SetItemButtonDesaturated(itemButton,1)
			else
				SetItemButtonNameFrameVertexColor(buybackButton, 0.5, 0.5, 0.5);
				SetItemButtonSlotVertexColor(buybackButton, 1.0, 1.0, 1.0);
				SetItemButtonTextureVertexColor(itemButton, 1.0, 1.0, 1.0);
				SetItemButtonNormalTextureVertexColor(itemButton, 1.0, 1.0, 1.0);
			end


		else
			SetItemButtonNameFrameVertexColor(buybackButton, 0.5, 0.5, 0.5);
			SetItemButtonSlotVertexColor(buybackButton,0.4, 0.4, 0.4);
			_G["MerchantItem"..i.."Name"]:SetText("");
			_G["MerchantItem"..i.."MoneyFrame"]:Hide();
			itemButton:Hide();
		end
	end	
	-- Hide all merchant related items
	MerchantRepairAllButton:Hide();
	MerchantRepairItemButton:Hide();
	MerchantBuyBackItem:Hide();
	MerchantPrevPageButton:Hide();
	MerchantNextPageButton:Hide();
	MerchantFrameBottomLeftBorder:Hide();
	MerchantFrameBottomRightBorder:Hide();
	MerchantRepairText:Hide();
	MerchantPageText:Hide();
	MerchantGuildBankRepairButton:Hide();
	
	-- Hide buyback related items
	MerchantItem11:Hide();
	MerchantItem12:Hide();
	BuybackFrameTopLeft:Hide();
	BuybackFrameTopRight:Hide();
	BuybackFrameBotLeft:Hide();
	BuybackFrameBotRight:Hide();
end


